@@ -38,6 +38,7 @@ from pathlib import Path
 from shutil import copyfile
 from typing import Any, Dict, List
 from typing import Callable, Optional, Tuple, TypeVar, Union
+from urllib.parse import unquote
 import struct
 import warnings
 
@@ -46,7 +47,7 @@ from dataclasses_json.core import _decode_dataclass
 from dataclasses_json.core import _ExtendedEncoder as JsonEncoder
 from deprecated import deprecated
 
-__version__ = "1.14.4"
+__version__ = "1.15.0"
 
 """
 About the GLTF2 file format:
@@ -636,6 +637,25 @@ class GLTF2(Property):
                           "Please open an issue at https://gitlab.com/dodgyville/pygltflib/issues")
         return uri_format
 
+    def get_data_from_buffer_uri(self, uri):
+        """
+        No matter how the buffer data is stored (the uri may be a long string, a file name or imply
+        a binary blob), strip off any headers and do any conversions are return a universal binary
+        blob for manipulation.
+        """
+        current_buffer_format = self.identify_uri(uri)
+
+        if current_buffer_format == BufferFormat.BINFILE:
+            data = self.load_file_uri(uri)
+        elif current_buffer_format == BufferFormat.DATAURI:
+            data = self.decode_data_uri(uri)
+        elif current_buffer_format == BufferFormat.BINARYBLOB:
+            data = self.binary_blob()
+        else:
+            return None
+
+        return data
+
     # noinspection PyPep8Naming
     def remove_bufferView(self, buffer_view_id):
         """
@@ -664,7 +684,6 @@ class GLTF2(Property):
 
         return bufferView
 
-
     def export_datauri_as_image_file(self, data_uri, name, destination, override=False, index=0):
         """ convert data uri to image file
             If destination is full path and file name, use that.
@@ -679,7 +698,7 @@ class GLTF2(Property):
             file_name = f"{index}{extension}"
         destination = Path(destination)
         if destination.is_dir():
-            image_path = destination / file_name
+            image_path = destination / unquote(file_name)
         else:  # assume filepath
             image_path = destination
         if image_path.is_file() and not override:
@@ -694,7 +713,7 @@ class GLTF2(Property):
     def export_fileuri_as_image_file(self, file_uri, destination, override=False):
         """ Export file uri as another image file (ie copy out of GLTF into own location) """
         path = getattr(self, "_path", Path())
-        image_path = Path(path / file_uri)
+        image_path = Path(path / unquote(file_uri))
         if not image_path.exists():
             warnings.warn(f"Unable to find image {image_path} for export.")
             return None
@@ -732,8 +751,9 @@ class GLTF2(Property):
         image = self.images[image_index]
         if image.uri and not image.uri.startswith('data:'):
             # already in file format
-            if not (destination_path / image.uri).exists():
-                warnings.warn(f"Image {image_index} is already stored in a file {destination_path / image.uri} but file"
+            image_path = destination_path / unquote(image.uri)
+            if not image_path.exists():
+                warnings.warn(f"Image {image_index} is already stored in a file {image_path} but file "
                               f"does not appear to exist.")
             return None
         elif image.bufferView is not None:
@@ -760,7 +780,13 @@ class GLTF2(Property):
                 return file_name
             return None
         elif image.uri.startswith('data:'):
-            file_name = self.export_datauri_as_image_file(image.uri, image.name, destination_path, override, image_index)
+            file_name = self.export_datauri_as_image_file(
+                image.uri,
+                image.name,
+                destination_path,
+                override,
+                image_index,
+            )
             return file_name
 
     def convert_images(self, image_format, path=None, override=False):
@@ -783,7 +809,7 @@ class GLTF2(Property):
                 if image.uri:  # either already in a datauri format, or in a file
                     if not image.uri.startswith('data:'):  # not in data format, so assume a file name
                         # data is stored in a file, so load into data uri
-                        image_path = Path(path / image.uri)
+                        image_path = Path(path / unquote(image.uri))
                         if not image_path.exists():
                             warnings.warn(f"Expected image file at {image_path} not found.")
                             continue
@@ -834,15 +860,11 @@ class GLTF2(Property):
             current_buffer_format = self.identify_uri(buffer.uri)
             if current_buffer_format == buffer_format:  # already in the format
                 continue
-
             if current_buffer_format == BufferFormat.BINFILE:
                 warnings.warn(f"Conversion will leave {buffer.uri} file orphaned since data is now in the GLTF object.")
-                data = self.load_file_uri(buffer.uri)
-            elif current_buffer_format == BufferFormat.DATAURI:
-                data = self.decode_data_uri(buffer.uri)
-            elif current_buffer_format == BufferFormat.BINARYBLOB:
-                data = self.binary_blob()
-            else:
+            data = self.get_data_from_buffer_uri(buffer.uri)
+
+            if not data:
                 return
 
             self.destroy_binary_blob()  # free up any binary blob floating around
@@ -855,7 +877,7 @@ class GLTF2(Property):
                 self.set_binary_blob(data)
                 buffer.uri = ''
             elif buffer_format == BufferFormat.DATAURI:
-                # convert buffer
+                # convert buffer to a data uri
                 data = base64.b64encode(data).decode('utf-8')
                 buffer.uri = f'{DATA_URI_HEADER}{data}'
             elif buffer_format == BufferFormat.BINFILE:
@@ -923,8 +945,8 @@ class GLTF2(Property):
                     primitive.attributes = attributes
         return result
 
-    def gltf_to_json(self) -> str:
-        return self.to_json(default=json_serial, indent="  ", allow_nan=False, skipkeys=True)
+    def gltf_to_json(self, separators=None, indent="  ") -> str:
+        return self.to_json(default=json_serial, indent=indent, allow_nan=False, skipkeys=True, separators=separators)
 
     def save_json(self, fname):
         path = Path(fname)
@@ -946,19 +968,20 @@ class GLTF2(Property):
         self.buffers = original_buffers  # restore buffers
         return True
 
-    def save_to_bytes(self):
-        # setup
-        buffer_blob = b''
-        original_buffer_views = copy.deepcopy(self.bufferViews)
-        original_buffers = copy.deepcopy(self.buffers)
+    def buffers_to_binary_blob(self):
+        """ Flatten all buffers into a single buffer """
+        buffer_blob = bytearray()
 
         offset = 0
-        new_buffer = Buffer()
         path = getattr(self, "_path", Path())
+        binary_blob: Optional[bytearray] = None
+
         for i, bufferView in enumerate(self.bufferViews):
             buffer = self.buffers[bufferView.buffer]
             if buffer.uri == '':  # assume loaded from glb binary file
-                data = self.binary_blob()
+                if binary_blob is None:
+                    binary_blob = self.binary_blob()
+                data = binary_blob
             elif buffer.uri.startswith("data"):
                 data = self.decode_data_uri(buffer.uri)
             elif Path(path, buffer.uri).is_file():
@@ -980,10 +1003,19 @@ class GLTF2(Property):
             bufferView.buffer = 0
             offset += byte_length
 
-        new_buffer.byteLength = len(buffer_blob)
-        self.buffers = [new_buffer]
+        return buffer_blob
 
-        json_blob = self.gltf_to_json().encode("utf-8")
+    def save_to_bytes(self):
+        # setup
+        original_buffer_views = copy.deepcopy(self.bufferViews)
+        original_buffers = copy.deepcopy(self.buffers)
+        new_buffer = Buffer()
+
+        buffer_blob = self.buffers_to_binary_blob()
+        new_buffer.byteLength = len(buffer_blob)
+
+        self.buffers = [new_buffer]
+        json_blob = self.gltf_to_json(separators=(',', ':'), indent=None).encode("utf-8")
 
         # pad each blob if needed
         if len(json_blob) % 4 != 0:
@@ -992,6 +1024,7 @@ class GLTF2(Property):
         version = struct.pack('<I', GLTF_VERSION)
         chunk_header_len = 8
         length = len(MAGIC) + len(version) + 4 + chunk_header_len * 2 + len(json_blob) + len(buffer_blob)
+
         self.bufferViews = original_buffer_views  # restore unpacked bufferViews
         self.buffers = original_buffers  # restore unpacked buffers
 
